;MJLJStimulator by MJM (c)2008
;Development kit -  Atmel STK500
;Processor - AVR ATTiny series - currently ATTiny13
;******************************************************************************
;Configuration Control
;
;Filename:		MJLJStimulator.asm
;Use:			Main entry file
;
;Version		Author		Date		Notes
;1.0			MJM			02/01/08	Initial Version	
;******************************************************************************

.include "tn13def.inc" 				;ATTiny13 definitions

;PIP output
.equ	PIP_OUT			= 1			;PB1 == PIP out

;******************************************************************************
									;Start of Flash memory
;******************************************************************************
.cseg
.org 0x0000 						;Interrupt handlers...
	rjmp 	INTH_RESET				; Reset Handler
	rjmp 	INTH_EXT_INT0			; IRQ0 Handler
	rjmp 	INTH_EXT_PCINT0			; Pin Change 0 Handler
	rjmp 	INTH_TIMER0_OVF			; Timer0 Overflow Handler
	rjmp 	INTH_EE_RDY				; EEPROM Ready Handler
	rjmp 	INTH_ANA_COMP			; Analog Comparator Handler
	rjmp 	INTH_TIM0_COMPA			; Timer0 Compare Match A Handler
	rjmp 	INTH_TIM0_COMPB			; Timer0 Compare Match B Handler
	rjmp 	INTH_WDT				; Watchdog Handler
	rjmp 	INTH_ADC				; ADC Conversion Complete Handler
;******************************************************************************
;include files
;******************************************************************************
.include "lib_initialise.asm"		;initialisation code

;******************************************************************************
INTH_RESET:							; Reset Handler
;******************************************************************************
	;set stack pointer
	ldi		r16, low (RAMEND)
	out		SPL, r16

	;call initialisation routine
	rcall	lib_initialise

loop:
	;loop forever

	;If ADC conversion in progress, skip...
	sbis	ADCSRA, ADSC
	;...else start ADC conversion
	sbi		ADCSRA, ADSC

	rjmp 	loop

;******************************************************************************
INTH_EXT_INT0:						; IRQ0 Handler
;******************************************************************************
	reti

;******************************************************************************
INTH_EXT_PCINT0:					; Pin Change 0 Handler
;******************************************************************************
	reti

;******************************************************************************
INTH_TIMER0_OVF:					; Timer0 Overflow Handler
;******************************************************************************

	;save registers
	push	r16
	in 		r16, SREG
	push	r16
	push	r17
	push	r18
	push	r19
	push	r20
	push	r21

	;reload timer value
	ldi		r16, 0xFF
	out 	TCNT0, r16

	;...increment timer loop count
	ldi		r20, 0x01
	ldi		r21, 0

	lds		r18, TimerLoopCount
	lds		r19, TimerLoopCount+1

	add		r18, r20
	adc		r19, r21

	;compare with loop target
	lds		r16, TimerLoopTarget
	lds		r17, TimerLoopTarget+1

	cp		r18, r16
	cpc		r19, r17

	brcc	toggle_output

	;store loop counter and exit
	sts		TimerLoopCount, r18
	sts		TimerLoopCount+1, r19
	rjmp	end_INTH_TIMER0_OVF

toggle_output:
	;get port value
	in 		r16, PinB
	;check if PIP on...
	andi 	r16, 1 << PIP_OUT
	;..if so turn it off
	breq	turn_pip_off

	;...otherwise turn PIP on...
	ldi 	r16, 0xFF - (1 << PIP_OUT)
	out 	PortB, r16
	;...and exit
	rjmp	reset_loop_counter
	 
turn_pip_off:
	;turn PIP off
	ldi 	r16, (0 << PIP_OUT)
	ldi 	r16, 0xFF
	out 	PortB, r16
 
reset_loop_counter:
	;reset loop counter and exit
	ldi		r18, 0
	sts		TimerLoopCount, r18
	sts		TimerLoopCount+1, r18
	rjmp	end_INTH_TIMER0_OVF

end_INTH_TIMER0_OVF:
	;restore registers
	pop		r21
	pop		r20
	pop		r19
	pop		r18
	pop		r17
	pop		r16
	out 	SREG, r16
	pop		r16

	reti

;******************************************************************************
INTH_EE_RDY:						; EEPROM Ready Handler
;******************************************************************************
	;unused
	reti

;******************************************************************************
INTH_ANA_COMP:						; Analog Comparator Handler
;******************************************************************************
	;unused
	reti

;******************************************************************************
INTH_TIM0_COMPA:					; Timer0 Compare Match A Handler
;******************************************************************************
	;unused
	reti

;******************************************************************************
INTH_TIM0_COMPB:					; Timer0 Compare Match B Handler
;******************************************************************************
	;unused
	reti

;******************************************************************************
INTH_WDT:							; Watchdog Handler
;******************************************************************************
	;unused
	reti

;******************************************************************************
INTH_ADC:							; ADC Conversion Complete Handler
;******************************************************************************

	;save registers
	push	r16
	in 		r16, SREG
	push	r16
	push	r17
	push	r18

	;get and store the ADC value
	in 		r18, ADCH
	ldi		r17, 0

	;find timer value by offsetting into table
	ldi		ZH, high (timer_cyl_4_start * 2)
	ldi		ZL, low (timer_cyl_4_start * 2)
	add		ZL, r18
	adc		ZH, r17
	add		ZL, r18
	adc		ZH, r17
	lpm		r16, Z+
	;store LSB
	sts		TimerLoopTarget, r16
	lpm		r16, Z
	;store MSB
	sts		TimerLoopTarget + 1, r16
		
end_INTH_ADC:
	;restore registers
	pop 	r18
	pop 	r17
	pop 	r16
	out 	SREG, r16
	pop		r16
	reti

;******************************************************************************
;timer values for each rpm value
;accessed by offset representing ADC value i.e. RPM pot position
;calculated using external Excel spreadsheet
;******************************************************************************
timer_cyl_4_start:
rpm500:		.dw 0x1194	;offset from 500rpm = 0 - keeps rpm at 500 for low pot position
			.dw 0x1194	;offset from 500rpm = 0 - keeps rpm at 500 for low pot position
			.dw 0x1194	;offset from 500rpm = 0 - keeps rpm at 500 for low pot position
			.dw 0x1194	;offset from 500rpm = 0 - keeps rpm at 500 for low pot position
			.dw 0x1194	;offset from 500rpm = 0 - keeps rpm at 500 for low pot position
			.dw 0x1194	;offset from 500rpm = 0 - keeps rpm at 500 for low pot position
;rpm531:	.dw 0x108D	;offset from 500rpm = 1 - not used
;rpm562:	.dw 0x0FA3	;offset from 500rpm = 2 - not used
;rpm593:	.dw 0x0ED2	;offset from 500rpm = 3 - not used
;rpm625:	.dw 0x0E10	;offset from 500rpm = 4 - not used
;rpm656:	.dw 0x0D66	;offset from 500rpm = 5 - not used
rpm687:		.dw 0x0CCB	;offset from 500rpm = 6
rpm718:		.dw 0x0C3E	;offset from 500rpm = 7
rpm750:		.dw 0x0BB8	;offset from 500rpm = 8
rpm781:		.dw 0x0B41	;offset from 500rpm = 9
rpm812:		.dw 0x0AD3	;offset from 500rpm = 10
rpm843:		.dw 0x0A6D	;offset from 500rpm = 11
rpm875:		.dw 0x0A0B	;offset from 500rpm = 12
rpm906:		.dw 0x09B3	;offset from 500rpm = 13
rpm937:		.dw 0x0961	;offset from 500rpm = 14
rpm968:		.dw 0x0914	;offset from 500rpm = 15
rpm1000:	.dw 0x08CA	;offset from 500rpm = 16
rpm1031:	.dw 0x0886	;offset from 500rpm = 17
rpm1062:	.dw 0x0847	;offset from 500rpm = 18
rpm1093:	.dw 0x080B	;offset from 500rpm = 19
rpm1125:	.dw 0x07D0	;offset from 500rpm = 20
rpm1156:	.dw 0x079A	;offset from 500rpm = 21
rpm1187:	.dw 0x0768	;offset from 500rpm = 22
rpm1218:	.dw 0x0737	;offset from 500rpm = 23
rpm1250:	.dw 0x0708	;offset from 500rpm = 24
rpm1281:	.dw 0x06DC	;offset from 500rpm = 25
rpm1312:	.dw 0x06B3	;offset from 500rpm = 26
rpm1343:	.dw 0x068B	;offset from 500rpm = 27
rpm1375:	.dw 0x0664	;offset from 500rpm = 28
rpm1406:	.dw 0x0640	;offset from 500rpm = 29
rpm1437:	.dw 0x061E	;offset from 500rpm = 30
rpm1468:	.dw 0x05FD	;offset from 500rpm = 31
rpm1500:	.dw 0x05DC	;offset from 500rpm = 32
rpm1531:	.dw 0x05BE	;offset from 500rpm = 33
rpm1562:	.dw 0x05A0	;offset from 500rpm = 34
rpm1593:	.dw 0x0584	;offset from 500rpm = 35
rpm1625:	.dw 0x0569	;offset from 500rpm = 36
rpm1656:	.dw 0x054F	;offset from 500rpm = 37
rpm1687:	.dw 0x0536	;offset from 500rpm = 38
rpm1718:	.dw 0x051E	;offset from 500rpm = 39
rpm1750:	.dw 0x0506	;offset from 500rpm = 40
rpm1781:	.dw 0x04EF	;offset from 500rpm = 41
rpm1812:	.dw 0x04DA	;offset from 500rpm = 42
rpm1843:	.dw 0x04C5	;offset from 500rpm = 43
rpm1875:	.dw 0x04B0	;offset from 500rpm = 44
rpm1906:	.dw 0x049C	;offset from 500rpm = 45
rpm1937:	.dw 0x048A	;offset from 500rpm = 46
rpm1968:	.dw 0x0477	;offset from 500rpm = 47
rpm2000:	.dw 0x0465	;offset from 500rpm = 48
rpm2031:	.dw 0x0454	;offset from 500rpm = 49
rpm2062:	.dw 0x0443	;offset from 500rpm = 50
rpm2093:	.dw 0x0433	;offset from 500rpm = 51
rpm2125:	.dw 0x0423	;offset from 500rpm = 52
rpm2156:	.dw 0x0414	;offset from 500rpm = 53
rpm2187:	.dw 0x0405	;offset from 500rpm = 54
rpm2218:	.dw 0x03F6	;offset from 500rpm = 55
rpm2250:	.dw 0x03E8	;offset from 500rpm = 56
rpm2281:	.dw 0x03DA	;offset from 500rpm = 57
rpm2312:	.dw 0x03CD	;offset from 500rpm = 58
rpm2343:	.dw 0x03C0	;offset from 500rpm = 59
rpm2375:	.dw 0x03B3	;offset from 500rpm = 60
rpm2406:	.dw 0x03A7	;offset from 500rpm = 61
rpm2437:	.dw 0x039B	;offset from 500rpm = 62
rpm2468:	.dw 0x0390	;offset from 500rpm = 63
rpm2500:	.dw 0x0384	;offset from 500rpm = 64
rpm2531:	.dw 0x0379	;offset from 500rpm = 65
rpm2562:	.dw 0x036E	;offset from 500rpm = 66
rpm2593:	.dw 0x0364	;offset from 500rpm = 67
rpm2625:	.dw 0x0359	;offset from 500rpm = 68
rpm2656:	.dw 0x034F	;offset from 500rpm = 69
rpm2687:	.dw 0x0345	;offset from 500rpm = 70
rpm2718:	.dw 0x033C	;offset from 500rpm = 71
rpm2750:	.dw 0x0332	;offset from 500rpm = 72
rpm2781:	.dw 0x0329	;offset from 500rpm = 73
rpm2812:	.dw 0x0320	;offset from 500rpm = 74
rpm2843:	.dw 0x0317	;offset from 500rpm = 75
rpm2875:	.dw 0x030F	;offset from 500rpm = 76
rpm2906:	.dw 0x0306	;offset from 500rpm = 77
rpm2937:	.dw 0x02FE	;offset from 500rpm = 78
rpm2968:	.dw 0x02F6	;offset from 500rpm = 79
rpm3000:	.dw 0x02EE	;offset from 500rpm = 80
rpm3031:	.dw 0x02E6	;offset from 500rpm = 81
rpm3062:	.dw 0x02DF	;offset from 500rpm = 82
rpm3093:	.dw 0x02D7	;offset from 500rpm = 83
rpm3125:	.dw 0x02D0	;offset from 500rpm = 84
rpm3156:	.dw 0x02C9	;offset from 500rpm = 85
rpm3187:	.dw 0x02C2	;offset from 500rpm = 86
rpm3218:	.dw 0x02BB	;offset from 500rpm = 87
rpm3250:	.dw 0x02B4	;offset from 500rpm = 88
rpm3281:	.dw 0x02AE	;offset from 500rpm = 89
rpm3312:	.dw 0x02A7	;offset from 500rpm = 90
rpm3343:	.dw 0x02A1	;offset from 500rpm = 91
rpm3375:	.dw 0x029B	;offset from 500rpm = 92
rpm3406:	.dw 0x0295	;offset from 500rpm = 93
rpm3437:	.dw 0x028F	;offset from 500rpm = 94
rpm3468:	.dw 0x0289	;offset from 500rpm = 95
rpm3500:	.dw 0x0283	;offset from 500rpm = 96
rpm3531:	.dw 0x027D	;offset from 500rpm = 97
rpm3562:	.dw 0x0278	;offset from 500rpm = 98
rpm3593:	.dw 0x0272	;offset from 500rpm = 99
rpm3625:	.dw 0x026D	;offset from 500rpm = 100
rpm3656:	.dw 0x0267	;offset from 500rpm = 101
rpm3687:	.dw 0x0262	;offset from 500rpm = 102
rpm3718:	.dw 0x025D	;offset from 500rpm = 103
rpm3750:	.dw 0x0258	;offset from 500rpm = 104
rpm3781:	.dw 0x0253	;offset from 500rpm = 105
rpm3812:	.dw 0x024E	;offset from 500rpm = 106
rpm3843:	.dw 0x0249	;offset from 500rpm = 107
rpm3875:	.dw 0x0245	;offset from 500rpm = 108
rpm3906:	.dw 0x0240	;offset from 500rpm = 109
rpm3937:	.dw 0x023B	;offset from 500rpm = 110
rpm3968:	.dw 0x0237	;offset from 500rpm = 111
rpm4000:	.dw 0x0232	;offset from 500rpm = 112
rpm4031:	.dw 0x022E	;offset from 500rpm = 113
rpm4062:	.dw 0x022A	;offset from 500rpm = 114
rpm4093:	.dw 0x0226	;offset from 500rpm = 115
rpm4125:	.dw 0x0221	;offset from 500rpm = 116
rpm4156:	.dw 0x021D	;offset from 500rpm = 117
rpm4187:	.dw 0x0219	;offset from 500rpm = 118
rpm4218:	.dw 0x0215	;offset from 500rpm = 119
rpm4250:	.dw 0x0211	;offset from 500rpm = 120
rpm4281:	.dw 0x020E	;offset from 500rpm = 121
rpm4312:	.dw 0x020A	;offset from 500rpm = 122
rpm4343:	.dw 0x0206	;offset from 500rpm = 123
rpm4375:	.dw 0x0202	;offset from 500rpm = 124
rpm4406:	.dw 0x01FF	;offset from 500rpm = 125
rpm4437:	.dw 0x01FB	;offset from 500rpm = 126
rpm4468:	.dw 0x01F8	;offset from 500rpm = 127
rpm4500:	.dw 0x01F4	;offset from 500rpm = 128
rpm4531:	.dw 0x01F1	;offset from 500rpm = 129
rpm4562:	.dw 0x01ED	;offset from 500rpm = 130
rpm4593:	.dw 0x01EA	;offset from 500rpm = 131
rpm4625:	.dw 0x01E6	;offset from 500rpm = 132
rpm4656:	.dw 0x01E3	;offset from 500rpm = 133
rpm4687:	.dw 0x01E0	;offset from 500rpm = 134
rpm4718:	.dw 0x01DD	;offset from 500rpm = 135
rpm4750:	.dw 0x01DA	;offset from 500rpm = 136
rpm4781:	.dw 0x01D7	;offset from 500rpm = 137
rpm4812:	.dw 0x01D4	;offset from 500rpm = 138
rpm4843:	.dw 0x01D1	;offset from 500rpm = 139
rpm4875:	.dw 0x01CE	;offset from 500rpm = 140
rpm4906:	.dw 0x01CB	;offset from 500rpm = 141
rpm4937:	.dw 0x01C8	;offset from 500rpm = 142
rpm4968:	.dw 0x01C5	;offset from 500rpm = 143
rpm5000:	.dw 0x01C2	;offset from 500rpm = 144
rpm5031:	.dw 0x01BF	;offset from 500rpm = 145
rpm5062:	.dw 0x01BC	;offset from 500rpm = 146
rpm5093:	.dw 0x01BA	;offset from 500rpm = 147
rpm5125:	.dw 0x01B7	;offset from 500rpm = 148
rpm5156:	.dw 0x01B4	;offset from 500rpm = 149
rpm5187:	.dw 0x01B2	;offset from 500rpm = 150
rpm5218:	.dw 0x01AF	;offset from 500rpm = 151
rpm5250:	.dw 0x01AD	;offset from 500rpm = 152
rpm5281:	.dw 0x01AA	;offset from 500rpm = 153
rpm5312:	.dw 0x01A8	;offset from 500rpm = 154
rpm5343:	.dw 0x01A5	;offset from 500rpm = 155
rpm5375:	.dw 0x01A3	;offset from 500rpm = 156
rpm5406:	.dw 0x01A0	;offset from 500rpm = 157
rpm5437:	.dw 0x019E	;offset from 500rpm = 158
rpm5468:	.dw 0x019B	;offset from 500rpm = 159
rpm5500:	.dw 0x0199	;offset from 500rpm = 160
rpm5531:	.dw 0x0197	;offset from 500rpm = 161
rpm5562:	.dw 0x0195	;offset from 500rpm = 162
rpm5593:	.dw 0x0192	;offset from 500rpm = 163
rpm5625:	.dw 0x0190	;offset from 500rpm = 164
rpm5656:	.dw 0x018E	;offset from 500rpm = 165
rpm5687:	.dw 0x018C	;offset from 500rpm = 166
rpm5718:	.dw 0x0189	;offset from 500rpm = 167
rpm5750:	.dw 0x0187	;offset from 500rpm = 168
rpm5781:	.dw 0x0185	;offset from 500rpm = 169
rpm5812:	.dw 0x0183	;offset from 500rpm = 170
rpm5843:	.dw 0x0181	;offset from 500rpm = 171
rpm5875:	.dw 0x017F	;offset from 500rpm = 172
rpm5906:	.dw 0x017D	;offset from 500rpm = 173
rpm5937:	.dw 0x017B	;offset from 500rpm = 174
rpm5968:	.dw 0x0179	;offset from 500rpm = 175
rpm6000:	.dw 0x0177	;offset from 500rpm = 176
rpm6031:	.dw 0x0175	;offset from 500rpm = 177
rpm6062:	.dw 0x0173	;offset from 500rpm = 178
rpm6093:	.dw 0x0171	;offset from 500rpm = 179
rpm6125:	.dw 0x016F	;offset from 500rpm = 180
rpm6156:	.dw 0x016E	;offset from 500rpm = 181
rpm6187:	.dw 0x016C	;offset from 500rpm = 182
rpm6218:	.dw 0x016A	;offset from 500rpm = 183
rpm6250:	.dw 0x0168	;offset from 500rpm = 184
rpm6281:	.dw 0x0166	;offset from 500rpm = 185
rpm6312:	.dw 0x0164	;offset from 500rpm = 186
rpm6343:	.dw 0x0163	;offset from 500rpm = 187
rpm6375:	.dw 0x0161	;offset from 500rpm = 188
rpm6406:	.dw 0x015F	;offset from 500rpm = 189
rpm6437:	.dw 0x015D	;offset from 500rpm = 190
rpm6468:	.dw 0x015C	;offset from 500rpm = 191
rpm6500:	.dw 0x015A	;offset from 500rpm = 192
rpm6531:	.dw 0x0159	;offset from 500rpm = 193
rpm6562:	.dw 0x0157	;offset from 500rpm = 194
rpm6593:	.dw 0x0155	;offset from 500rpm = 195
rpm6625:	.dw 0x0154	;offset from 500rpm = 196
rpm6656:	.dw 0x0152	;offset from 500rpm = 197
rpm6687:	.dw 0x0150	;offset from 500rpm = 198
rpm6718:	.dw 0x014F	;offset from 500rpm = 199
rpm6750:	.dw 0x014D	;offset from 500rpm = 200
rpm6781:	.dw 0x014C	;offset from 500rpm = 201
rpm6812:	.dw 0x014A	;offset from 500rpm = 202
rpm6843:	.dw 0x0149	;offset from 500rpm = 203
rpm6875:	.dw 0x0147	;offset from 500rpm = 204
rpm6906:	.dw 0x0146	;offset from 500rpm = 205
rpm6937:	.dw 0x0144	;offset from 500rpm = 206
rpm6968:	.dw 0x0143	;offset from 500rpm = 207
rpm7000:	.dw 0x0141	;offset from 500rpm = 208
rpm7031:	.dw 0x0140	;offset from 500rpm = 209
rpm7062:	.dw 0x013F	;offset from 500rpm = 210
rpm7093:	.dw 0x013D	;offset from 500rpm = 211
rpm7125:	.dw 0x013C	;offset from 500rpm = 212
rpm7156:	.dw 0x013A	;offset from 500rpm = 213
rpm7187:	.dw 0x0139	;offset from 500rpm = 214
rpm7218:	.dw 0x0138	;offset from 500rpm = 215
rpm7250:	.dw 0x0136	;offset from 500rpm = 216
rpm7281:	.dw 0x0135	;offset from 500rpm = 217
rpm7312:	.dw 0x0134	;offset from 500rpm = 218
rpm7343:	.dw 0x0132	;offset from 500rpm = 219
rpm7375:	.dw 0x0131	;offset from 500rpm = 220
rpm7406:	.dw 0x0130	;offset from 500rpm = 221
rpm7437:	.dw 0x012F	;offset from 500rpm = 222
rpm7468:	.dw 0x012D	;offset from 500rpm = 223
rpm7500:	.dw 0x012C	;offset from 500rpm = 224
rpm7531:	.dw 0x012B	;offset from 500rpm = 225
rpm7562:	.dw 0x012A	;offset from 500rpm = 226
rpm7593:	.dw 0x0128	;offset from 500rpm = 227
rpm7625:	.dw 0x0127	;offset from 500rpm = 228
rpm7656:	.dw 0x0126	;offset from 500rpm = 229
rpm7687:	.dw 0x0125	;offset from 500rpm = 230
rpm7718:	.dw 0x0124	;offset from 500rpm = 231
rpm7750:	.dw 0x0122	;offset from 500rpm = 232
rpm7781:	.dw 0x0121	;offset from 500rpm = 233
rpm7812:	.dw 0x0120	;offset from 500rpm = 234
rpm7843:	.dw 0x011F	;offset from 500rpm = 235
rpm7875:	.dw 0x011E	;offset from 500rpm = 236
rpm7906:	.dw 0x011D	;offset from 500rpm = 237
rpm7937:	.dw 0x011B	;offset from 500rpm = 238
rpm7968:	.dw 0x011A	;offset from 500rpm = 239
rpm8000:	.dw 0x0119	;offset from 500rpm = 240
rpm8031:	.dw 0x0118	;offset from 500rpm = 241
rpm8062:	.dw 0x0117	;offset from 500rpm = 242
rpm8093:	.dw 0x0116	;offset from 500rpm = 243
rpm8125:	.dw 0x0115	;offset from 500rpm = 244
rpm8156:	.dw 0x0114	;offset from 500rpm = 245
rpm8187:	.dw 0x0113	;offset from 500rpm = 246
rpm8218:	.dw 0x0112	;offset from 500rpm = 247
rpm8250:	.dw 0x0111	;offset from 500rpm = 248
rpm8281:	.dw 0x0110	;offset from 500rpm = 249
rpm8312:	.dw 0x010F	;offset from 500rpm = 250
rpm8343:	.dw 0x010E	;offset from 500rpm = 251
rpm8375:	.dw 0x010D	;offset from 500rpm = 252
rpm8406:	.dw 0x010C	;offset from 500rpm = 253
rpm8437:	.dw 0x010B	;offset from 500rpm = 254
rpm8468:	.dw 0x010A	;offset from 500rpm = 255

;******************************************************************************
;.data segment variables
;******************************************************************************
.dseg
.org SRAM_START

TimerLoopCount			: .byte 0x02
TimerLoopTarget			: .byte 0x02
