;******************************************************************************
;Configuration Control
;
;Filename:	lib_initialise.asm
;Use:		initialisation routine
;
;Version		Author		Date		Notes
;1.0			MJM			02/01/08	Initial Version	
;******************************************************************************

;******************************************************************************
lib_initialise:						;initialisation code
;******************************************************************************

	;PORTB initialisation...
	;...PB1 (PIP_OUT) is output
	ldi		r16, (1 << PIP_OUT)
	out		ddrb, r16

	;set PIP low
	ldi 	r16, 0xFF - (1 << PIP_OUT)
	out		portb, r16

	;other SRAM initialise...to one
	;nothing.....

	;other SRAM initialise...to zero
	ldi		r16, 0
	sts		TimerLoopCount, r16
	sts		TimerLoopCount + 1, r16

	;other initialise...
	;10000rpm = 6.00mS on = 0x02EE
	; 6000rpm = 9.98mS on = 0x04E0
	; 1184rpm =50.60mS on = 0x18BF
	;set initial rpm
	ldi		r16, 0xEE
	sts		TimerLoopTarget, r16
	ldi		r16, 0x02
	sts		TimerLoopTarget + 1, r16

	;ADC initialise
	;Vcc is reference
	;left adjust result
	;single ended input on ADC1 (PB2)
	ldi		r16, (0 << REFS0) | (1 <<ADLAR) | (0 <<MUX1) | (1 << MUX0)
	out		ADMUX, r16
	;ADC is enabled
	;ADC interrupt is enabled
	;prescaler is 128
	ldi		r16, (1 << ADEN) | (1 <<ADIE) | (1 <<ADPS2) | (1 <<ADPS1) | (1 <<ADPS0) 
	out		ADCSRA, r16

interrupt_initialise:
	;initialise external interrupts

	;set timer 0 overflow interrupt enable 0
	ldi		r16, (1<< TOIE0)
	out 	TIMSK0, r16

	ldi		r16, 0xFF
	out		TCNT0, r16

	;set timer 0 prescaler to CK/1024
	;ldi		r16, (1 << CS02 | 0 << CS01 | 1 << CS00)
	;set timer 0 prescaler to CK/256
	;ldi		r16, (1 << CS02 | 0 << CS01 | 0 << CS00)
	;set timer 0 prescaler to CK/64
	ldi		r16, (0 << CS02 | 1 << CS01 | 1 << CS00)
	;set timer 0 prescaler to CK/8
	;ldi		r16, (0 << CS02 | 1 << CS01 | 0 << CS00)
	;set timer 0 prescaler to CK/1 i.e no prescaling
	;ldi		r16, (0 << CS02 | 0 << CS01 | 1 << CS00)
	out 	TCCR0B, r16

	;enable interrupts
	sei

	ret
